﻿namespace UltraGridDragAndDropWithScroll
{
using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        public const int GridRows = 50;
        public Form1()
        {
            this.InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraGrid1.DataSource = this.GetData(GridRows);
            this.ultraGrid1.AllowDrop = true;
        }

        private DataSet GetData(int rows)
        {
            DataSet ds = new DataSet();
            DataTable dt = ds.Tables.Add();

            dt.Columns.Add("Id", typeof(int));
            dt.Columns.Add("Name", typeof(string));

            for (int row = 0; row < rows; row++)
            {
                int id = row;
                string name = "Row #" + row;
                dt.Rows.Add(new object[] { id, name });
            }

            return ds;

        }

        private void UltraGrid_SelectionDrag(object sender, CancelEventArgs e)
        {
            this.ultraGrid1.DoDragDrop(this.ultraGrid1.Selected.Rows, DragDropEffects.Move);
        }

        private void UltraGrid_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
            UltraGrid grid = sender as UltraGrid;
            if (grid == null)
            {
                return;
            }

            Point point = grid.PointToClient(new Point(e.X, e.Y));
            if (point.Y < 20)
            {
                // Scroll up.
                this.ultraGrid1.ActiveRowScrollRegion.Scroll(RowScrollAction.LineUp);
            }
            else if (point.Y > grid.Height - 20)
            {    // Scroll down.
                this.ultraGrid1.ActiveRowScrollRegion.Scroll(RowScrollAction.LineDown);
            }
        }

        private void UltraGrid_DragDrop(object sender, DragEventArgs e)
        {
            int dropIndex;

            UIElement uieOver = ultraGrid1.DisplayLayout.UIElement.ElementFromPoint(ultraGrid1.PointToClient(new Point(e.X, e.Y)));
            UltraGridRow ugrOver = uieOver.GetContext(typeof(UltraGridRow), true) as UltraGridRow;
            
            if (ugrOver != null)
            {
                dropIndex = ugrOver.Index;
                SelectedRowsCollection SelRows = (SelectedRowsCollection)e.Data.GetData(typeof(SelectedRowsCollection)) as SelectedRowsCollection;

                foreach (UltraGridRow aRow in SelRows)
                {
                    ultraGrid1.Rows.Move(aRow, dropIndex);
                }
            }
        }
    }
}
